@@ -4,10 +4,10 @@ from copy import deepcopy, copy
 import traceback
 
 from pyperry import errors
+from pyperry import callbacks
 from pyperry.relation import Relation
 from pyperry.adapter.abstract_adapter import AbstractAdapter
-from pyperry.association import BelongsTo, HasMany, HasOne
-from pyperry.association import Association
+from pyperry.association import BelongsTo, HasMany, HasOne, Association
 from pyperry.field import Field
 from pyperry.scope import Scope, DefaultScope
 
@@ -23,7 +23,7 @@ class BaseMeta(type):
 
     defined_models = {}
 
-    def __new__(mcs, name, bases, dict_):
+    def __new__(mcs, name, bases, class_dict):
         """
         Called any time a new Base class is created using this metaclass
 
@@ -33,40 +33,44 @@ class BaseMeta(type):
         """
 
         # Create the new class
-        new = type.__new__(mcs, name, bases, dict_)
-
-        # Create a default primary_key value
-        if not hasattr(new, '_primary_key'):
-            new._primary_key = 'id'
-
-        # Create fresh adapter dict
-        new._adapters = {}
-
-        # Define any fields set during class definition
-        if not hasattr(new, 'defined_fields'):
-            new.defined_fields = set()
-        else:
-            new.defined_fields = deepcopy(new.defined_fields)
-
-        # Define any associations set during class definition
-        if not hasattr(new, 'defined_associations'):
-            new.defined_associations = {}
-        else:
-            new.defined_associations = deepcopy(new.defined_associations)
+        cls = type.__new__(mcs, name, bases, class_dict)
 
         # Track model definitions by name
         if not mcs.defined_models.has_key(name):
             mcs.defined_models[name] = []
-        mcs.defined_models[name].append(new)
+        mcs.defined_models[name].append(cls)
 
-        return new
+        return cls
 
     _relation_delegates = (Relation.singular_query_methods +
                 Relation.plural_query_methods +
-                ['modifiers', 'all', 'first', 'find'])
+                ['modifiers', 'all', 'first', 'find', 'update_all',
+                'delete_all'])
 
     def __init__(cls, name, bases, class_dict):
         """Class has been created now setup additional needs"""
+        # Create a default primary_key value
+        if not hasattr(cls, '_primary_key'):
+            cls._primary_key = 'id'
+
+        # Define any fields set during class definition
+        if not hasattr(cls, 'defined_fields'):
+            cls.defined_fields = set()
+        else:
+            cls.defined_fields = copy(cls.defined_fields)
+
+        # Define any associations set during class definition
+        if not hasattr(cls, 'defined_associations'):
+            cls.defined_associations = {}
+        else:
+            cls.defined_associations = deepcopy(cls.defined_associations)
+
+        if not hasattr(cls, 'callback_manager'):
+            cls.callback_manager = callbacks.CallbackManager()
+        else:
+            cls.callback_manager = callbacks.CallbackManager(
+                    cls.callback_manager )
+
         # Force calling of __setattr__ for each defined attribute for special
         # attribute handling
         for key in class_dict.keys():
@@ -100,6 +104,7 @@ class BaseMeta(type):
             - %L{Association}
             - %L{DefaultScope} (special case of Scope)
             - %L{Scope}
+            - %L{Callback}
 
         This method is also called for each attribute set during class creation
         allowing for common behavior to be applied whether set during class
@@ -123,6 +128,8 @@ class BaseMeta(type):
             value.__name__ = key
             if not hasattr(cls, 'scopes'): cls.scopes = {}
             cls.scopes[key] = value
+        elif isinstance(value, callbacks.Callback):
+            cls.callback_manager.register(value)
 
         type.__setattr__(cls, key, value)
 
@@ -379,6 +386,7 @@ class Base(object):
     """
 
     __metaclass__ = BaseMeta
+    _relation_class = Relation
 
     def __init__(self, fields=None, new_record=True, **kwargs):
         """
@@ -399,17 +407,21 @@ class Base(object):
             fields = {}
         fields.update(kwargs)
 
-        self.fields = self.default_fields()
+        self.saved = None
+        self.errors = {}
+        self._frozen = False
         self.new_record = new_record
 
+        self.callback_manager.trigger(callbacks.before_load, self)
+
+        self.fields = self.default_fields()
+
         if self.new_record:
             self.set_fields(fields)
         else:
             self.set_raw_fields(fields)
 
-        self.saved = None
-        self.errors = {}
-        self._frozen = False
+        self.callback_manager.trigger(callbacks.after_load, self)
 
     #{ Dict-like access
     def __getitem__(self, key):
@@ -574,7 +586,7 @@ class Base(object):
                 self[field] = fields[field]
 
 
-    def save(self):
+    def save(self, run_callbacks=True):
         """
         Save the current value of the model's data fields through the write
         adapter.
@@ -600,10 +612,32 @@ class Base(object):
             raise errors.PersistenceError(
                     "cannot save model without a primary key value")
 
-        self.last_writer_response = self.writer(model=self, mode='write')
-        return self.last_writer_response.success
+        create_operation = self.new_record
 
-    def update(self):
+        # Before Callbacks
+        if run_callbacks:
+            self.callback_manager.trigger(callbacks.before_save, self)
+
+            if create_operation:
+                self.callback_manager.trigger(callbacks.before_create, self)
+            else:
+                self.callback_manager.trigger(callbacks.before_update, self)
+
+        # Run the save
+        self.writer.last_response = self.writer(model=self, mode='write')
+
+        # After callbacks
+        if run_callbacks:
+            if create_operation:
+                self.callback_manager.trigger(callbacks.after_create, self)
+            else:
+                self.callback_manager.trigger(callbacks.after_update, self)
+
+            self.callback_manager.trigger(callbacks.after_save, self)
+
+        return self.writer.last_response.success
+
+    def update(self, **kwargs):
         """
         Save the record if it is not a new_record and raise PersistenceError
         otherwise.
@@ -619,9 +653,9 @@ class Base(object):
                     "update() must only be called on an existing record but "
                     "new_record attribute is True" )
 
-        return self.save()
+        return self.save(**kwargs)
 
-    def create(self):
+    def create(self, **kwargs):
         """
         Save the record iff it is a new_record and raise PersistenceError
         otherwise.
@@ -637,7 +671,7 @@ class Base(object):
                     "create() must only be called on a new record but "
                     "new_record attribute is False" )
 
-        return self.save()
+        return self.save(**kwargs)
 
     def update_fields(self, fields=None, **kwargs):
         """
@@ -664,7 +698,7 @@ class Base(object):
 
         return self.save()
 
-    def delete(self):
+    def delete(self, run_callbacks=True):
         """
         Removes this model from the data store
 
@@ -692,8 +726,15 @@ class Base(object):
             raise errors.PersistenceError(
                     'cannot delete a model without a primary key value')
 
-        self.last_writer_response = self.writer(model=self, mode='delete')
-        return self.last_writer_response.success
+        if run_callbacks:
+            self.callback_manager.trigger(callbacks.before_delete, self)
+
+        self.writer.last_response = self.writer(model=self, mode='delete')
+
+        if run_callbacks:
+            self.callback_manager.trigger(callbacks.after_delete, self)
+
+        return self.writer.last_response.success
 
     #}
 
@@ -766,7 +807,7 @@ class Base(object):
 
         """
         if not hasattr(cls, '_relation') or cls._relation.klass != cls:
-            cls._relation = Relation(cls)
+            cls._relation = cls._relation_class(cls)
         return cls._relation
 
     @classmethod
